// Learn more about Tauri commands at https://tauri.app/develop/calling-rust/

// Standard library imports
use std::path::Path;
use std::env;

// Serde for serialization/deserialization

// Tauri related imports

// Internal modules
pub mod models;
pub mod services;
pub mod commands;
pub mod crawlers;
pub mod platform_matcher;

// Import our models
pub use models::server::{ServerType, ServerInfo, ScanResult, ScanProgress};
pub use models::plugin::{Plugin, PluginMeta};
pub use models::repository::{RepositorySource, RepositoryPlugin, PotentialPluginMatch};

// Import our services
pub use services::http::HttpClient;
pub use services::plugin_scanner::{scan_server_directory, perform_scan, extract_plugin_metadata, calculate_file_hash, is_file_locked};
pub use services::update_manager::{check_for_plugin_updates, check_single_plugin_update, backup_plugin, replace_plugin, normalize_version, compare_plugin_versions};

// Import our commands
pub use commands::plugin_commands::*;
pub use commands::scan_commands::*;
pub use commands::util_commands::*;

// Import our crawlers
pub use crawlers::HangarCrawler;
pub use crawlers::SpigotMCCrawler;
pub use crawlers::ModrinthCrawler;
pub use crawlers::GitHubCrawler;
pub use crawlers::Repository;

// Import platform matchers
pub use platform_matcher::{get_compatible_modrinth_loaders, is_version_compatible_with_server};

use futures::future::{BoxFuture, FutureExt}; // Import necessary future types

/// Search for plugins in repositories
pub async fn lib_search_plugins_in_repositories(
    query: &str,
    repositories: Vec<RepositorySource>
) -> Result<Vec<RepositoryPlugin>, String> {
    // Check for empty query
    if query.is_empty() {
        return Err("Search query cannot be empty".to_string());
    }

    // Check for empty repository list
    if repositories.is_empty() {
        return Err("No repositories specified for search".to_string());
    }

    // Create a list to store tasks (boxed futures)
    let mut tasks: Vec<BoxFuture<Result<(String, Vec<RepositoryPlugin>), (String, String)>>> = Vec::new();

    // Add tasks for each repository
    for repo in repositories {
        match repo {
            RepositorySource::HangarMC => {
                let crawler = crawlers::HangarCrawler::new();
                let query_owned = query.to_string();
                let task = async move {
                 match crawler.search(&query_owned).await {
                    Ok(repo_results) => Ok((crawler.get_repository_name(), repo_results)),
                        Err(e) => Err((crawler.get_repository_name(), e.to_string()))
                    }
                }.boxed(); // Box the future
                tasks.push(task);
            },
            RepositorySource::SpigotMC => {
                let crawler = crawlers::SpigotMCCrawler::new();
                let query_owned = query.to_string();
                let task = async move {
                    match crawler.search(&query_owned).await {
                        Ok(repo_results) => Ok((crawler.get_repository_name(), repo_results)),
                        Err(e) => Err((crawler.get_repository_name(), e.to_string()))
                    }
                }.boxed(); // Box the future
                tasks.push(task);
            },
            RepositorySource::Modrinth => {
                let crawler = crawlers::ModrinthCrawler::new();
                let query_owned = query.to_string();
                let task = async move {
                    match crawler.search(&query_owned).await {
                        Ok(repo_results) => Ok((crawler.get_repository_name(), repo_results)),
                        Err(e) => Err((crawler.get_repository_name(), e.to_string()))
                    }
                }.boxed(); // Box the future
                tasks.push(task);
            },
            RepositorySource::GitHub => {
                let crawler = crawlers::GitHubCrawler::new();
                let query_owned = query.to_string();
                let task = async move {
                    match crawler.search(&query_owned).await {
                        Ok(repo_results) => Ok((crawler.get_repository_name(), repo_results)),
                        Err(e) => Err((crawler.get_repository_name(), e.to_string()))
                    }
                }.boxed(); // Box the future
                tasks.push(task);
            },
            _ => {
                eprintln!("Repository {:?} not supported for search", repo);
            }
        }
    }

    // Execute all tasks concurrently
    let results = futures::future::join_all(tasks).await;

    // Collect successful results
    let mut plugins = Vec::new();
    let mut errors = Vec::new();

    for result in results {
         match result {
            Ok((_, repo_plugins)) => {
                plugins.extend(repo_plugins);
            },
            Err((repo, error)) => {
                errors.push(format!("[{}] {}", repo, error));
            }
        }
    }

    // Check if we found any results
    if plugins.is_empty() {
        if !errors.is_empty() {
            return Err(errors.join("; "));
                } else {
            return Ok(Vec::new());
        }
    }

    Ok(plugins)
}

/// Generate search variations for a plugin name using advanced techniques
fn generate_search_variations(plugin_name: &str) -> Vec<String> {
    let mut variations = Vec::new();

    // Add original name
    variations.push(plugin_name.to_string());

    // Convert to lowercase
    let name_lower = plugin_name.to_lowercase();
    if name_lower != plugin_name {
        variations.push(name_lower.clone());
    }

    // Normalize whitespace and separators
    let normalized = normalize_separators(&name_lower);
    if normalized != name_lower {
        variations.push(normalized.clone());
    }

    // Extract core name by removing common prefixes/suffixes
    let core_name = extract_core_name(&normalized);
    if !core_name.is_empty() && core_name != normalized {
        variations.push(core_name.clone());
    }

    // Handle common abbreviations and full forms
    variations.extend(expand_abbreviations(&core_name));

    // Handle word re-ordering for multi-word plugin names
    if core_name.contains(' ') {
        let words: Vec<&str> = core_name.split_whitespace().collect();
        if words.len() > 1 && words.len() <= 4 { // Only try reordering for 2-4 word names
            // Add key words without modifiers
            let significant_words: Vec<&str> = words.iter()
                .filter(|w| w.len() > 3 && !is_common_modifier(w))
                .copied()
                .collect();

            if significant_words.len() > 0 && significant_words.len() < words.len() {
                variations.push(significant_words.join(" "));
            }
        }
    }

    // Special handling for plugins that often include terms like "API", "Core", etc.
    if core_name.contains("api") {
        variations.push(core_name.replace("api", "").trim().to_string());
    }

    if core_name.contains("core") {
        variations.push(core_name.replace("core", "").trim().to_string());
    }

    // Handle plugins that might be searched by acronym
    // (e.g., "WorldEditCUI" -> "WECUI")
    if let Some(acronym) = generate_acronym(&core_name) {
        if acronym.len() >= 2 {
            variations.push(acronym);
        }
    }

    // Add variations with common prefixes/suffixes removed
    // This is more comprehensive than the original
    let prefixes = ["plugin", "mc", "minecraft", "bukkit", "spigot", "paper", "forge", "fabric"];
    let suffixes = ["plugin", "spigot", "bukkit", "paper", "mc", "forge", "fabric", "addon", "plus", "+", "mod"];

    for prefix in prefixes.iter() {
        // Check for space or separator after prefix
        let prefix_patterns = [
            format!("{} ", prefix),      // "prefix "
            format!("{}-", prefix),      // "prefix-"
            format!("{}:", prefix),      // "prefix:"
            format!("{}core", prefix),   // "prefixcore"
        ];

        for pattern in prefix_patterns {
            if name_lower.starts_with(&pattern) {
                variations.push(name_lower[pattern.len()..].trim().to_string());
            }
        }
    }

    for suffix in suffixes.iter() {
        // Check for space or separator before suffix
        let suffix_patterns = [
            format!(" {}", suffix),      // " suffix"
            format!("-{}", suffix),      // "-suffix"
            format!(":{}", suffix),      // ":suffix"
            format!("for{}", suffix),    // "forsuffix"
        ];

        for pattern in suffix_patterns {
            if name_lower.ends_with(&pattern) {
                variations.push(name_lower[0..name_lower.len() - pattern.len()].trim().to_string());
            }
        }
    }

    // Remove duplicates and empty strings
    variations.retain(|s| !s.is_empty());
    variations.sort();
    variations.dedup();

    // Debug: print variations
    println!("Generated {} search variations for '{}':", variations.len(), plugin_name);
    for (i, v) in variations.iter().enumerate() {
        println!("  {}: '{}'", i+1, v);
    }

    variations
}

/// Normalize separators in a plugin name
fn normalize_separators(name: &str) -> String {
    let mut result = name.to_lowercase();

    // Replace various separators with spaces
    result = result.replace('-', " ");
    result = result.replace('_', " ");
    result = result.replace('.', " ");

    // Normalize spaces (multiple spaces to single space)
    while result.contains("  ") {
        result = result.replace("  ", " ");
    }

    result.trim().to_string()
}

/// Extract the core name by removing common prefixes/suffixes
fn extract_core_name(name: &str) -> String {
    let mut result = name.to_lowercase();

    // Common prefixes to remove
    let prefixes = [
        "the ", "a ", "an ", "official ", "premium ", "free ",
        "minecraft ", "mc ", "bukkit ", "spigot ", "paper ",
    ];

    // Common suffixes to remove
    let suffixes = [
        " plugin", " mod", " addon", " plus", " ++", " +", " pro",
        " free", " premium", " lite", " light", " full", " version",
    ];

    // Remove prefixes
    for prefix in &prefixes {
        if result.starts_with(prefix) {
            result = result[prefix.len()..].to_string();
        }
    }

    // Remove suffixes
    for suffix in &suffixes {
        if result.ends_with(suffix) {
            result = result[..result.len() - suffix.len()].to_string();
        }
    }

    // Remove version numbers at the end
    // Examples: "Plugin 1.0", "Plugin v2", "Plugin 2.0.1"
    if let Some(index) = result.rfind(|c: char| c == ' ' || c == 'v') {
        let potential_version = &result[index+1..];
        if potential_version.chars().all(|c| c.is_digit(10) || c == '.') {
            result = result[..index].to_string();
        }
    }

    result.trim().to_string()
}

/// Generate possible expansions of abbreviations
fn expand_abbreviations(name: &str) -> Vec<String> {
    let mut expansions = Vec::new();

    // Common abbreviations in plugin names
    let abbreviations = [
        ("gui", vec!["graphical user interface"]),
        ("ui", vec!["user interface"]),
        ("api", vec!["application programming interface", "interface"]),
        ("cmds", vec!["commands"]),
        ("cmd", vec!["command"]),
        ("mgr", vec!["manager"]),
        ("mgmt", vec!["management"]),
        ("auth", vec!["authentication", "authenticator"]),
        ("eco", vec!["economy"]),
        ("tp", vec!["teleport", "teleporter"]),
        ("inv", vec!["inventory"]),
        ("prot", vec!["protection", "protector"]),
        ("chat", vec!["chatter"]),
        ("admin", vec!["administrator", "administration"]),
        ("perms", vec!["permissions"]),
        ("essentials", vec!["ess"]),
        ("warp", vec!["warps"]),
        ("spawn", vec!["spawner", "spawns"]),
        ("we", vec!["worldedit", "world edit"]),
        ("wb", vec!["worldborder", "world border"]),
        ("wg", vec!["worldguard", "world guard"]),
        ("lb", vec!["logblock", "log block"]),
        ("gui", vec!["menu", "interface"]),
    ];

    let words: Vec<&str> = name.split_whitespace().collect();

    // Try replacing abbreviations in each word
    for (abbr, expansions_list) in &abbreviations {
        for (i, word) in words.iter().enumerate() {
            if word == abbr {
                for expansion in expansions_list {
                    // Replace this word with the expansion
                    let mut new_words = words.clone();
                    new_words[i] = expansion;
                    expansions.push(new_words.join(" "));
                }
            }
        }
    }

    // Also try abbreviating common terms
    for (abbr, expansions_list) in &abbreviations {
        for expansion in expansions_list {
            if name.contains(expansion) {
                expansions.push(name.replace(expansion, abbr));
            }
        }
    }

    expansions
}

/// Generate an acronym from a plugin name (e.g., "WorldEditCUI" -> "WECUI")
fn generate_acronym(name: &str) -> Option<String> {
    // Method 1: CamelCase splitting
    let mut acronym = String::new();
    let chars: Vec<char> = name.chars().collect();

    // Special handling for CamelCase
    for i in 0..chars.len() {
        if i == 0 {
            // Always include first character
            acronym.push(chars[i].to_ascii_uppercase());
        } else if chars[i].is_uppercase() && (i == 0 || !chars[i-1].is_uppercase()) {
            // Include uppercase letters that start a new word
            acronym.push(chars[i]);
        }
    }

    // Method 2: Word splitting
    if acronym.len() <= 1 {
        acronym.clear();
        let words: Vec<&str> = name.split(|c: char| c == ' ' || c == '-' || c == '_').collect();

        // Take first letter of each word
        for word in words {
            if !word.is_empty() {
                if let Some(c) = word.chars().next() {
                    acronym.push(c.to_ascii_uppercase());
                }
            }
        }
    }

    if acronym.len() >= 2 {
        Some(acronym)
    } else {
        None
    }
}

/// Check if a word is a common modifier that doesn't add much meaning
fn is_common_modifier(word: &str) -> bool {
    let modifiers = [
        "the", "a", "an", "of", "for", "with", "and", "or", "in", "on", "by",
        "to", "from", "lite", "pro", "premium", "free", "plus"
    ];

    modifiers.contains(&word.to_lowercase().as_str())
}

/// Search for plugin variations
pub async fn search_with_variations(plugin_name: &str, repositories: &[RepositorySource]) -> Result<Vec<RepositoryPlugin>, String> {
    let variations = generate_search_variations(plugin_name);
    let mut all_results = Vec::new();

    for variation in variations {
        match lib_search_plugins_in_repositories(&variation, repositories.to_vec()).await {
            Ok(results) => {
                    all_results.extend(results);
            },
            Err(e) => {
                println!("Error searching for variation '{}': {}", variation, e);
                // Continue with other variations even if this one fails
            }
        }
    }

    // Remove duplicates by plugin ID and repository
    all_results.sort_by(|a, b| {
        let a_key = format!("{:?}:{}", a.repository, a.id);
        let b_key = format!("{:?}:{}", b.repository, b.id);
        a_key.cmp(&b_key)
    });

    all_results.dedup_by(|a, b| {
        a.id == b.id && a.repository == b.repository
    });

    Ok(all_results)
}

/// Get plugin details from a repository
pub async fn lib_get_plugin_details_from_repository(
    plugin_id: &str,
    repository: RepositorySource,
    server_type: Option<&ServerType>
) -> Result<RepositoryPlugin, String> {
    match repository {
        RepositorySource::HangarMC => {
            let crawler = crawlers::HangarCrawler::new();
            crawler.get_plugin_details(plugin_id).await
                .map_err(|e| format!("Failed to get plugin details from HangarMC: {}", e))
        },
        RepositorySource::SpigotMC => {
            let crawler = crawlers::SpigotMCCrawler::new();
            crawler.get_plugin_details(plugin_id).await
                .map_err(|e| format!("Failed to get plugin details from SpigotMC: {}", e))
        },
        RepositorySource::Modrinth => {
            let crawler = crawlers::ModrinthCrawler::new();

            // Use server type aware version if provided
            if let Some(server_type) = server_type {
                crawler.get_plugin_details_with_server_type(plugin_id, Some(server_type)).await
                    .map_err(|e| format!("Failed to get plugin details from Modrinth: {}", e))
    } else {
                crawler.get_plugin_details(plugin_id).await
                    .map_err(|e| format!("Failed to get plugin details from Modrinth: {}", e))
            }
        },
        RepositorySource::GitHub => {
            let crawler = crawlers::GitHubCrawler::new();
            crawler.get_plugin_details(plugin_id).await
                .map_err(|e| format!("Failed to get plugin details from GitHub: {}", e))
        },
        _ => Err(format!("Repository source {:?} not supported for plugin details", repository))
    }
}

/// Download a plugin from a repository
pub async fn lib_download_plugin_from_repository(
    plugin_id: &str,
    version: &str,
    repository: RepositorySource,
    destination: &str,
    server_type: Option<&ServerType>
) -> Result<String, String> {
    match repository {
        RepositorySource::HangarMC => {
            let crawler = crawlers::HangarCrawler::new();
            crawler.download_plugin(plugin_id, version, Path::new(destination)).await
                .map_err(|e| format!("Failed to download plugin from HangarMC: {}", e))
        },
        RepositorySource::SpigotMC => {
            let crawler = crawlers::SpigotMCCrawler::new();
            crawler.download_plugin(plugin_id, version, Path::new(destination)).await
                .map_err(|e| format!("Failed to download plugin from SpigotMC: {}", e))
        },
        RepositorySource::Modrinth => {
            let crawler = crawlers::ModrinthCrawler::new();

            // Use server type aware version if provided
            if let Some(server_type) = server_type {
                crawler.download_plugin_with_server_type(plugin_id, version, Path::new(destination), Some(server_type)).await
                    .map_err(|e| format!("Failed to download plugin from Modrinth: {}", e))
            } else {
                crawler.download_plugin(plugin_id, version, Path::new(destination)).await
                    .map_err(|e| format!("Failed to download plugin from Modrinth: {}", e))
            }
        },
        RepositorySource::GitHub => {
            let crawler = crawlers::GitHubCrawler::new();
            crawler.download_plugin(plugin_id, version, Path::new(destination)).await
                .map_err(|e| format!("Failed to download plugin from GitHub: {}", e))
        },
        _ => Err(format!("Repository source {:?} not supported for downloads", repository))
    }
}

// Tauri v2 plugin initialization
#[cfg_attr(mobile, tauri::mobile_entry_point)]
pub fn run() {
  tauri::Builder::default()
    .plugin(tauri_plugin_shell::init())
    .plugin(tauri_plugin_dialog::init())
    .plugin(tauri_plugin_fs::init())
    .setup(|app| {
      if cfg!(debug_assertions) {
        app.handle().plugin(
          tauri_plugin_log::Builder::default()
            .level(log::LevelFilter::Info)
            .build(),
        )?;
      }
      Ok(())
    })
    .invoke_handler(tauri::generate_handler![
      // Plugin discovery commands
      scan_server_dir,
      scan_server_dir_sync,

      // Plugin repository commands
      search_plugins,
      get_plugin_details,

      // Update commands
      update_plugin,
      check_plugin_updates,
      check_single_plugin_update_command,
      backup_plugin_command,

      // Plugin management commands
      download_plugin,
      set_plugin_repository,
      get_plugin_versions,
      load_plugin_data,
      save_plugin_data,

      // Utility commands
      get_potential_plugin_matches,
      compare_versions,
      is_plugin_compatible,
      get_app_version,
      greet
    ])
    .run(tauri::generate_context!())
    .expect("error while running tauri application");
}
