use semver::{Version, VersionReq};
use regex::Regex;

/// Normalize a version string to be semver compatible
pub fn normalize_version(version_str: &str) -> String {
    // Handle empty strings or "Latest"
    if version_str.is_empty() || version_str.eq_ignore_ascii_case("latest") {
        return "0.0.0".to_string(); // Default version if none provided
    }

    // Remove 'v' prefix if present
    let cleaned = version_str.trim_start_matches(|c| c == 'v' || c == 'V');

    // Remove double 'v' prefixes (like "vv1.0.0")
    let cleaned = if cleaned.starts_with("v") || cleaned.starts_with("V") {
        cleaned.trim_start_matches(|c| c == 'v' || c == 'V')
    } else {
        cleaned
    };

    cleaned.to_string()
}

/// Sanitize version string for comparison by removing platform suffixes
fn sanitize_version_for_comparison(version_str: &str) -> String {
    // First normalize the version
    let normalized = normalize_version(version_str);

    // Remove common platform-specific suffixes
    let platform_suffixes = [
        "-paper", "-spigot", "-bukkit", "-forge", "-fabric", "-neoforge",
        "-sponge", "-velocity", "-waterfall", "-bungeecord", "-quilt"
    ];

    let mut result = normalized.to_string();

    // Case-insensitive suffix removal
    for suffix in platform_suffixes.iter() {
        let suffix_lower = suffix.to_lowercase();
        let version_lower = result.to_lowercase();

        if version_lower.contains(&suffix_lower) {
            // Find the position of the suffix (case-insensitive)
            if let Some(pos) = version_lower.find(&suffix_lower) {
                // Remove the suffix (with original casing)
                result = result[0..pos].to_string();
            }
        }
    }

    // Remove any build metadata (anything after +)
    if let Some(plus_pos) = result.find('+') {
        result = result[0..plus_pos].to_string();
    }

    // Handle snapshot versions with build numbers
    if result.contains("-SNAPSHOT") || result.contains("-snapshot") {
        // Extract just the version part before any build info
        if let Some(snapshot_pos) = result.to_lowercase().find("-snapshot") {
            result = result[0..snapshot_pos].to_string();
        }
    }

    // Normalize dev build formats that use dash-separated numbers
    let build_regex = Regex::new(r"-build\d+").unwrap();
    result = build_regex.replace_all(&result, "").to_string();

    // Handle other common non-numeric suffixes
    let common_suffixes = ["-RELEASE", "-dev", "-final", "-stable"];
    for suffix in common_suffixes.iter() {
        let suffix_lower = suffix.to_lowercase();
        let version_lower = result.to_lowercase();

        if version_lower.ends_with(&suffix_lower) {
            let suffix_len = suffix_lower.len();
            result = result[0..result.len() - suffix_len].to_string();
        }
    }

    // If we've removed everything, return the original normalized version
    if result.is_empty() {
        return normalized;
    }

    // Make sure it ends with at least one digit for semver parsing
    if !result.chars().last().map_or(false, |c| c.is_ascii_digit()) {
        if let Some(last_digit_pos) = result.rfind(|c: char| c.is_ascii_digit()) {
            result = result[0..=last_digit_pos].to_string();
        }
    }

    // If we've removed version numbers entirely, return the original
    if !result.chars().any(|c| c.is_ascii_digit()) {
        return normalized;
    }

    result
}

/// Extracts platform suffix from a version string
pub fn extract_platform_suffix(version_str: &str) -> Option<String> {
    // Platform suffixes to check for
    let platform_suffixes = [
        "-paper", "-spigot", "-bukkit", "-forge", "-fabric", "-neoforge",
        "-sponge", "-velocity", "-waterfall", "-bungeecord", "-quilt"
    ];

    let version_lower = version_str.to_lowercase();

    for suffix in platform_suffixes.iter() {
        let suffix_lower = suffix.to_lowercase();
        if version_lower.contains(&suffix_lower) {
            // Return the actual platform with original case
            return Some(suffix_lower[1..].to_string()); // Remove the leading '-'
        }
    }

    None
}

/// Determine if a version is a pre-release (snapshot, beta, etc.)
fn is_prerelease_version(version_str: &str) -> bool {
    let version_lower = version_str.to_lowercase();

    // Check for common pre-release indicators
    version_lower.contains("-snapshot") ||
    version_lower.contains("-alpha") ||
    version_lower.contains("-beta") ||
    version_lower.contains("-pre") ||
    version_lower.contains("-rc") ||
    version_lower.contains("dev") ||
    version_lower.contains("test") ||
    version_lower.contains("nightly")
}

/// Compare two plugin versions to determine if an update is available
/// Returns true if repo_str represents a newer version than installed_str
pub fn compare_plugin_versions(installed_str: &str, repo_str: &str) -> bool {
    // Special case: identical strings are never upgrades
    if installed_str == repo_str {
        return false;
    }

    // Extract platform suffixes
    let installed_platform = extract_platform_suffix(installed_str);
    let repo_platform = extract_platform_suffix(repo_str);

    // If platforms differ and both are specified, it's not considered an upgrade
    // (we don't want to suggest forge versions for paper plugins)
    if let (Some(installed_p), Some(repo_p)) = (&installed_platform, &repo_platform) {
        if installed_p != repo_p {
            println!("Platforms differ: {} vs {}, not an upgrade", installed_p, repo_p);
            return false;
        }
    }

    // Check for downgrades from release to prerelease
    let installed_is_prerelease = is_prerelease_version(installed_str);
    let repo_is_prerelease = is_prerelease_version(repo_str);

    // Don't consider a pre-release version an upgrade from a stable version
    if !installed_is_prerelease && repo_is_prerelease {
        println!("Not upgrading from release {} to pre-release {}", installed_str, repo_str);
        return false;
    }

    // Sanitize versions for comparison by removing platform-specific suffixes
    let sanitized_installed = sanitize_version_for_comparison(installed_str);
    let sanitized_repo = sanitize_version_for_comparison(repo_str);

    // Log for debugging
    println!("Comparing versions: '{}'({}') vs '{}'('{}')",
             installed_str, sanitized_installed, repo_str, sanitized_repo);

    // Try to parse as semver
    match (Version::parse(&sanitized_installed), Version::parse(&sanitized_repo)) {
        (Ok(installed), Ok(repo)) => {
            // Properly formatted semver comparison
            println!("  Using semver comparison: {} vs {}", installed, repo);
            repo > installed
        },
        _ => {
            // Fallback to more sophisticated string comparison for non-semver versions
            if sanitized_installed == sanitized_repo {
                // Same base version, check if repo has a higher build number or qualifier
                compare_version_qualifiers(installed_str, repo_str)
            } else {
                // Try numeric component-by-component comparison
                compare_version_components(&sanitized_installed, &sanitized_repo)
            }
        }
    }
}

/// Compare version qualifiers and build numbers when base versions are the same
fn compare_version_qualifiers(installed_str: &str, repo_str: &str) -> bool {
    // Try to extract build numbers
    let installed_build = extract_build_number(installed_str);
    let repo_build = extract_build_number(repo_str);

    if let (Some(i_build), Some(r_build)) = (installed_build, repo_build) {
        return r_build > i_build;
    }

    // If qualifiers differ, use a rank-based system
    let installed_rank = get_qualifier_rank(installed_str);
    let repo_rank = get_qualifier_rank(repo_str);

    if installed_rank != repo_rank {
        return repo_rank > installed_rank;
    }

    // Default case - not considered an upgrade
    false
}

/// Assign rank values to different version qualifiers
fn get_qualifier_rank(version_str: &str) -> i32 {
    let lower_str = version_str.to_lowercase();

    if lower_str.contains("alpha") { return 10; }
    if lower_str.contains("beta") { return 20; }
    if lower_str.contains("rc") { return 30; }
    if lower_str.contains("pre") { return 40; }
    if lower_str.contains("snapshot") { return 50; }
    if lower_str.contains("nightly") { return 60; }
    if lower_str.contains("dev") { return 70; }
    if lower_str.contains("release") { return 100; }

    // Default for stable releases
    90
}

/// Extract build number from a version string
fn extract_build_number(version_str: &str) -> Option<i32> {
    // Try to find patterns like "build123" or "-b123" or ".123"
    let build_patterns = [
        Regex::new(r"build(\d+)").ok()?,
        Regex::new(r"-b(\d+)").ok()?,
        Regex::new(r"\.(\d+)$").ok()?,
        Regex::new(r"-(\d+)$").ok()?,
    ];

    for pattern in &build_patterns {
        if let Some(captures) = pattern.captures(version_str) {
            if let Some(build_match) = captures.get(1) {
                if let Ok(build) = build_match.as_str().parse::<i32>() {
                    return Some(build);
                }
            }
        }
    }

    None
}

/// Compare version strings component by component
fn compare_version_components(version1: &str, version2: &str) -> bool {
    // Split versions into numeric and non-numeric parts
    let v1_parts: Vec<&str> = version1.split(|c: char| !c.is_ascii_digit()).filter(|s| !s.is_empty()).collect();
    let v2_parts: Vec<&str> = version2.split(|c: char| !c.is_ascii_digit()).filter(|s| !s.is_empty()).collect();

    // Compare corresponding numeric parts
    let max_parts = v1_parts.len().max(v2_parts.len());

    for i in 0..max_parts {
        let n1 = v1_parts.get(i).and_then(|s| s.parse::<i32>().ok()).unwrap_or(0);
        let n2 = v2_parts.get(i).and_then(|s| s.parse::<i32>().ok()).unwrap_or(0);

        if n2 > n1 {
            return true;  // Version 2 is higher
        }
        if n1 > n2 {
            return false; // Version 1 is higher
        }
    }

    // If all numeric parts are equal, compare by string length
    // (Consider more segments to be a higher version, like 1.2.3 > 1.2)
    v2_parts.len() > v1_parts.len()
}

/// Extract version pattern from a string
pub fn extract_version_pattern(input: &str) -> Option<String> {
    // Look for version pattern like 1.19.2 in string
    let version_pattern = Regex::new(r"(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)").unwrap();
    if let Some(captures) = version_pattern.captures(input) {
        if let Some(version_match) = captures.get(0) {
            return Some(version_match.as_str().to_string());
        }
    }
    None
}

/// Check if a plugin version is compatible with a specific Minecraft version
pub fn is_version_compatible(plugin_version: &str, minecraft_versions: &Vec<String>) -> bool {
    // If no versions specified, assume compatible
    if minecraft_versions.is_empty() {
        return true;
    }

    // Check if the Minecraft version is in the list of supported versions
    for supported_version in minecraft_versions {
        if plugin_version == supported_version {
            return true;
        }

        // Try to parse the Minecraft version
        if let Ok(mc_version) = semver::Version::parse(plugin_version) {
            // Try to parse as a version requirement
            if let Ok(req) = semver::VersionReq::parse(supported_version) {
                if req.matches(&mc_version) {
                    return true;
                }
            }
        }

        // If version formats are incompatible, make best guess
        if plugin_version.contains(supported_version) || supported_version.contains(plugin_version) {
            return true;
        }
    }

    // No compatibility match found
    false
}