import React, { useState, useMemo } from 'react';
import { usePluginActions } from '../../../hooks/usePluginActions';
import { usePluginContext } from '../../../context/PluginContext/usePluginContext';
import { Plugin } from '../../../types/plugin.types';
import { PluginItem } from '../PluginItem/PluginItem';
import Button from '../../common/Button/Button';
import './PluginList.css';

export const PluginList: React.FC = () => {
  const { plugins, isCheckingUpdates } = usePluginActions();
  const { showPluginDetails } = usePluginContext();
  const [searchTerm, setSearchTerm] = useState<string>('');
  const [sortBy, setSortBy] = useState<'name' | 'version' | 'update'>('name');
  const [sortOrder, setSortOrder] = useState<'asc' | 'desc'>('asc');
  const [filterUpdates, setFilterUpdates] = useState<boolean>(false);

  // Advanced search with fuzzy matching and relevance scoring
  const filteredPlugins = useMemo(() => {
    if (!searchTerm.trim() && !filterUpdates) {
      return plugins;
    }

    // Filter by updates if that filter is active
    let results = filterUpdates ? plugins.filter(plugin => plugin.has_update) : plugins;

    if (!searchTerm.trim()) {
      return results;
    }

    const searchLower = searchTerm.toLowerCase().trim();
    const searchTerms = searchLower.split(/\s+/);

    // If using multiple search terms, match plugins that contain all terms
    if (searchTerms.length > 1) {
      return results.filter(plugin => {
        // Create a searchable text combining all relevant plugin fields
        const searchableText = [
          plugin.name,
          plugin.version,
          plugin.description || '',
          plugin.authors?.join(' ') || '',
          plugin.website || '',
          plugin.main_class || '',
        ].join(' ').toLowerCase();

        // Plugin must match all search terms to be included
        return searchTerms.every(term => searchableText.includes(term));
      });
    }

    // For single term search, score each plugin for relevance
    const scoredPlugins = results.map(plugin => {
      let score = 0;

      // Exact name match is highest priority
      if (plugin.name.toLowerCase() === searchLower) {
        score += 100;
      }
      // Name starts with search term
      else if (plugin.name.toLowerCase().startsWith(searchLower)) {
        score += 80;
      }
      // Name contains search term
      else if (plugin.name.toLowerCase().includes(searchLower)) {
        score += 60;
      }

      // Check if search term is an acronym of the plugin name
      // E.g., "WE" for "WorldEdit"
      if (searchLower.length >= 2 && isAcronymMatch(searchLower, plugin.name)) {
        score += 70;
      }

      // Secondary matches in other fields
      if (plugin.description?.toLowerCase().includes(searchLower)) {
        score += 40;
      }

      if (plugin.authors?.some(author => author.toLowerCase().includes(searchLower))) {
        score += 50;
      }

      if (plugin.main_class?.toLowerCase().includes(searchLower)) {
        score += 30;
      }

      if (plugin.version.toLowerCase().includes(searchLower)) {
        score += 20;
      }

      if (plugin.website?.toLowerCase().includes(searchLower)) {
        score += 20;
      }

      // Tags or categories (if available in your data model)
      if (plugin.repository_source?.toString().toLowerCase().includes(searchLower)) {
        score += 30;
      }

      return { plugin, score };
    });

    // Filter plugins that have at least some relevance and sort by score
    const relevantPlugins = scoredPlugins
      .filter(item => item.score > 0)
      .sort((a, b) => b.score - a.score)
      .map(item => item.plugin);

    return relevantPlugins;
  }, [plugins, searchTerm, filterUpdates]);

  // Memoize sorted plugins for performance
  const sortedPlugins = useMemo(() => {
    return [...filteredPlugins].sort((a, b) => {
      let comparison = 0;

      switch (sortBy) {
        case 'name':
          comparison = a.name.localeCompare(b.name);
          break;
        case 'version':
          comparison = a.version.localeCompare(b.version);
          break;
        case 'update':
          // Sort by update status (plugins with updates first)
          comparison = (b.has_update ? 1 : 0) - (a.has_update ? 1 : 0);
          break;
      }

      return sortOrder === 'asc' ? comparison : -comparison;
    });
  }, [filteredPlugins, sortBy, sortOrder]);

  // Function to check if search is an acronym of plugin name
  const isAcronymMatch = (acronym: string, fullName: string): boolean => {
    // Extract capital letters or the first letter of each word
    const words = fullName.split(/[\s-_]+/);

    // Method 1: First letter of each word
    const firstLetters = words.map(word => word.charAt(0).toLowerCase()).join('');
    if (firstLetters === acronym) {
      return true;
    }

    // Method 2: Capital letters in a camel case name
    const capitals = fullName.replace(/[^A-Z]/g, '').toLowerCase();
    if (capitals === acronym) {
      return true;
    }

    // For shorter search terms, check if it's a substring of the first letters
    if (acronym.length >= 2 && acronym.length <= 3 && firstLetters.includes(acronym)) {
      return true;
    }

    return false;
  };

  const handleSort = (criteria: 'name' | 'version' | 'update') => {
    if (criteria === sortBy) {
      // Toggle sort order if clicking the same criteria
      setSortOrder(prevOrder => prevOrder === 'asc' ? 'desc' : 'asc');
    } else {
      // Set new criteria and default to ascending
      setSortBy(criteria);
      setSortOrder('asc');
    }
  };

  const toggleUpdateFilter = () => {
    setFilterUpdates(!filterUpdates);
  };

  const clearFilters = () => {
    setSearchTerm('');
    setFilterUpdates(false);
  };

  // Get stats for UI feedback
  const updateCount = plugins.filter(p => p.has_update).length;

  return (
    <div className="plugin-list-container">
      <div className="plugin-list-header">
        <div className="search-and-filter">
          <input
            type="text"
            className="search-input"
            placeholder="Search plugins by name, author, description..."
            value={searchTerm}
            onChange={(e) => setSearchTerm(e.target.value)}
          />
          <div className="filter-options">
            <Button
              variant={filterUpdates ? "primary" : "secondary"}
              size="small"
              onClick={toggleUpdateFilter}
            >
              {filterUpdates ? "Showing Updates" : "Show Updates Only"}
              {updateCount > 0 && `(${updateCount})`}
            </Button>
            {(searchTerm || filterUpdates) && (
              <Button
                variant="secondary"
                size="small"
                onClick={clearFilters}
              >
                Clear Filters
              </Button>
            )}
          </div>
        </div>

        <div className="sort-controls">
          <span className="sort-label">Sort by:</span>
          <Button
            variant={sortBy === 'name' ? 'primary' : 'secondary'}
            size="small"
            onClick={() => handleSort('name')}
          >
            Name {sortBy === 'name' && (sortOrder === 'asc' ? '↑' : '↓')}
          </Button>
          <Button
            variant={sortBy === 'version' ? 'primary' : 'secondary'}
            size="small"
            onClick={() => handleSort('version')}
          >
            Version {sortBy === 'version' && (sortOrder === 'asc' ? '↑' : '↓')}
          </Button>
          <Button
            variant={sortBy === 'update' ? 'primary' : 'secondary'}
            size="small"
            onClick={() => handleSort('update')}
          >
            Update Status {sortBy === 'update' && (sortOrder === 'asc' ? '↓' : '↑')}
          </Button>
        </div>
      </div>

      <div className="plugin-list">
        {sortedPlugins.length > 0 ? (
          sortedPlugins.map(plugin => (
            <PluginItem
              key={plugin.file_path}
              plugin={plugin}
              onSelect={showPluginDetails}
            />
          ))
        ) : (
          <div className="no-results">
            <p>No plugins match your search criteria.</p>
            <Button variant="secondary" onClick={clearFilters}>Clear All Filters</Button>
          </div>
        )}
      </div>
    </div>
  );
};

export default PluginList;